package com.duoqio.actuatorshiro.shiro.autoconfigure;

import java.util.HashMap;
import java.util.Map;

import org.apache.shiro.authc.credential.HashedCredentialsMatcher;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.duoqio.actuatorshiro.shiro.realm.SecurityShiroRealm;
import com.duoqio.actuatorshiro.shiro.util.SaltUtil;

/**
 * Shiro configuration
 */
@Configuration
public class ShiroAutoConfigure {

	/**
	 * Control access
	 * @param securityManager
	 * @return
	 */
    @Bean
    public ShiroFilterFactoryBean shirFilter(org.apache.shiro.mgt.SecurityManager securityManager) {
		ShiroFilterFactoryBean shiroFilterFactoryBean = new ShiroFilterFactoryBean();
		shiroFilterFactoryBean.setSecurityManager(securityManager);

		Map<String, String> filterChainDefinitionMap = new HashMap<String, String>();
		shiroFilterFactoryBean.setUnauthorizedUrl("/unauthc");
		shiroFilterFactoryBean.setSuccessUrl("/home/index");
		//Define some intercepting urls
		filterChainDefinitionMap.put("/*", "anon");
		filterChainDefinitionMap.put("/authc/index", "authc");
		filterChainDefinitionMap.put("/authc/admin", "roles[admin]");
		filterChainDefinitionMap.put("/authc/renewable", "perms[Create,Update]");
		filterChainDefinitionMap.put("/authc/removable", "perms[Delete]");
		shiroFilterFactoryBean.setFilterChainDefinitionMap(filterChainDefinitionMap);
        return shiroFilterFactoryBean;
    }

    /**
     * 
     * Matches the token used by the user to log in and the user information stored in the database
     * @return
     */
    @Bean
    public HashedCredentialsMatcher hashedCredentialsMatcher() {
        HashedCredentialsMatcher hashedCredentialsMatcher = new HashedCredentialsMatcher();
        hashedCredentialsMatcher.setHashAlgorithmName(SaltUtil.ALGORITHM_NAME); // Hash algorithm
        hashedCredentialsMatcher.setHashIterations(SaltUtil.HASH_ITERATIONS); // Hash number
        return hashedCredentialsMatcher;
    }

    @Bean
    public SecurityShiroRealm shiroRealm() {
    	SecurityShiroRealm shiroRealm = new SecurityShiroRealm();
        shiroRealm.setCredentialsMatcher(hashedCredentialsMatcher()); 
        return shiroRealm;
    }

    @Bean
    public org.apache.shiro.mgt.SecurityManager securityManager() {
        DefaultWebSecurityManager securityManager = new DefaultWebSecurityManager();
        securityManager.setRealm(shiroRealm());
        return securityManager;
    }

    @Bean
    public SaltUtil saltUtil() {
        return new SaltUtil();
    }
	
}
